/*--------------------------------
 * 
 * Jerome.Gouzy@toulouse.inra.fr
 * Created: 20140407
 * 
 * $Id: sequenceset.c 436 2014-08-10 08:52:47Z gouzy $
 * 
 ---------------------------------*/

// uthash Copyright (c) 2005-2014, Troy D. Hanson  http://troydhanson.github.com/uthash/ All rights reserved.

#include "sequenceset.h"

UT_icd linkcov_icd = {sizeof(linkcov_t), NULL, NULL, NULL};

// je fais +1 car je commence les id de sequence a 1
int MallocSeqSet(sequenceset_t***seqset, seqid_t nb_max_seq)
{
	sequenceset_t **seqsettmp=(sequenceset_t **)malloc(sizeof(sequenceset_t *)*(nb_max_seq+1));
	assert(seqsettmp!=NULL);
	seqid_t i;
	for(i=0;i<=nb_max_seq;i++) 
		seqsettmp[i]=NULL;
	*seqset=seqsettmp;
	return(TRUE);
}

/*
void DelSeqLinks(sequenceset_t**seqset, seqid_t nb_max_seq)
{
	seqid_t i;
	for(i=0;i<=nb_max_seq;i++) 
	{
		if ( seqset[i] != NULL && seqset[i]->a_links != NULL )
		{
			utarray_free(seqset[i]->a_links);
			seqset[i]->a_links=NULL;
		}
	}
}
*/

// soit je compte les obsoletes soit les ok 

seqid_t GetSequenceSetCount(sequenceset_t** seqset, seqid_t nb_max_seq,int what)
{
	seqid_t count=0;
	seqid_t i;
	for(i=0;i<=nb_max_seq;i++)
		if ( seqset[i]!=NULL )
		{
			if ( (what == OBSOLETE && seqset[i]->status == '1' ) || ( what == PAIRS && seqset[i]->status == '0' ) || ( what == VLR_CAND && seqset[i]->status == '5' ) )
				count++;
			else if ( what == LR && seqset[i]->lr != NULL )
				count++;
		}
	return(count);
}

// soit tout soit que les obsolete
void DeleteSequenceSet(sequenceset_t*** seqset, seqid_t nb_max_seq,int obsolete_only )
{
	sequenceset_t **seqsettmp = *seqset;
	if ( seqsettmp != NULL )
	{
		seqid_t i=0;
		for(i=0;i<=nb_max_seq;i++)
			if ( seqsettmp[i] != NULL )
			{
				//fprintf(stderr,"%u, %c, %s\n",i,seqset[i]->status,seqset[i]->sequence);
				if ( (obsolete_only && seqsettmp[i]->status == '1' ) || ! obsolete_only )
					DelSeq(seqsettmp,i,FALSE);
			}
		if ( ! obsolete_only )
		{
			free(seqsettmp);
			*seqset=NULL;
		}
	}
}

void DelSeq(sequenceset_t **seqset,seqid_t id,int partial)
{
	if ( seqset[id] != NULL )
	{
		free(seqset[id]->sequence);
		seqset[id]->sequence=NULL;
		/*
		if ( seqset[id]->a_links != NULL )
		{
			utarray_free(seqset[id]->a_links);
			seqset[id]->a_links=NULL;
		}
		*/
		if ( ! partial )
		{
			DelSeqLR(seqset,id);
			free(seqset[id]);
			seqset[id]=NULL;
		}
	}
}

void DelSeqLR(sequenceset_t **seqset,seqid_t id)
{
	if ( seqset[id] != NULL && seqset[id]->lr != NULL )
	{
		free(seqset[id]->lr);
		seqset[id]->lr=NULL;
	}
}

void DelSeqHub(sequenceset_t **seqset,seqid_t id,int what)
{
	switch(what)
	{
		case PAIRS:
		case OBSOLETE:
			DelSeq(seqset,id,FALSE);
			break;
		case LR:
			DelSeqLR(seqset,id);
			break;
	}
}

char *GetSeq(sequenceset_t **seqset,seqid_t id)
{
	if( seqset[id] != NULL )
		return(seqset[id]->sequence);
	return NULL;
}

char *GetSeqLR(sequenceset_t **seqset,seqid_t id)
{
	if( seqset[id] != NULL )
		return(seqset[id]->lr); // qui peut etre NULL
	return NULL;
}

char *GetSeqHub(sequenceset_t **seqset,seqid_t id,int what)
{
	switch(what)
	{
		case PAIRS:
		case OBSOLETE:
			return(GetSeq(seqset,id));
		case LR:
			return(GetSeqLR(seqset,id));
		default:
			return(NULL);
	}
}


// je ne veux compter que lorsque le status change

int SetSeqStatus(sequenceset_t **seqset,seqid_t id,char status)
{
	if ( seqset[id] != NULL && seqset[id]->status != status )
	{
		seqset[id]->status=status;
		return(TRUE);
	}
	return(FALSE);
}

// je recode mais il ne faut plus que je l'utilise
/*
int MoveSeq(sequenceset_t **seqset_src,sequenceset_t **seqset_dst,seqid_t id)
{
	if ( seqset_src[id] != NULL )
	{
		SetSeq(seqset_dst,id,seqset_src[id]->sequence); // je passe par la fonction, plus sur
		DelSeq(seqset_src,id);
		return(TRUE);
	}
	else
		return(FALSE);
}
*/

void LowerCase(sequenceset_t **seqset,seqid_t id,int start,int end,char strand,int what)
{
	char *seq = GetSeqHub(seqset,id,what);
	if ( seq != NULL )
	{
		int lstart = start;
		int lend   = end;
		if ( strand == '-' )
		{
			int len = strlen(seq);
			lstart = len-end+1;
			lend   = len-start+1;
		}
		LowerCaseSeq(seq,lstart,lend);
//		fprintf(stderr,"%u, %d-%d %c#, delta_code=%d\n",id,lstart,lend,strand,delta_code);
	}
}


/*
void SetQual(sequenceset_t **seqset,seqid_t id,float qualscore)
{
	if ( seqset[id] != NULL )
		seqset[id]->qualscore=qualscore;
}
*/


/*
void PrintQual(sequenceset_t **seqset,seqid_t id)
{
	if ( seqset[id] != NULL )
		fprintf(stderr,"%u Q=%f\n",id,seqset[id]->qualscore);
}
*/

void SetSeq(sequenceset_t **seqset,seqid_t id,char *seq)
{
	if ( seqset[id] != NULL )
		DelSeq(seqset,id,FALSE);
	seqset[id]=(sequenceset_t *)malloc(sizeof(sequenceset_t));
	assert(seqset[id]!=NULL);
	seqset[id]->sequence=(char *)malloc(strlen(seq)+1);
	assert(seqset[id]->sequence!=NULL);
	strcpy(seqset[id]->sequence,seq);
//	seqset[id]->qualscore=0;
	seqset[id]->lr=NULL;
	/*
	seqset[id]->a_links=NULL;
	utarray_new(seqset[id]->a_links,&linkcov_icd); // je le cree immediatement
	*/
	seqset[id]->status='0'; // sequence ok
}

// je veux pouvoir ne rajouter que la LR (pour les VLR)
void SetSeqLR(sequenceset_t **seqset,seqid_t id,char *seq)
{
	if ( seqset[id] == NULL )
	{
		seqset[id]=(sequenceset_t *)malloc(sizeof(sequenceset_t));
		assert(seqset[id]!=NULL);
		seqset[id]->sequence=NULL;
//		seqset[id]->qualscore=0;
		seqset[id]->lr=NULL;
//		seqset[id]->a_links=NULL;
		seqset[id]->status='0';
	}
	assert(seqset[id]->lr==NULL); 
	seqset[id]->lr=(char *)malloc(strlen(seq)+1);
	assert(seqset[id]->lr!=NULL);
	strcpy(seqset[id]->lr,seq);
}
